pro trace_make_vignette, time0, time1, vdata, vmask, index=index, $
   wavelen=wavelen, despike=despike, $
   savedbase=savedbase, overwrite=overwrite, outdir=outdir,   $
   minimages=minimages, maximages=maximages, $
   debug=debug, quiet=quiet
;
;+
;   Name:  trace_make_vignette
;
;   Purpose: generate wave dependent vignette mask, optionaly save->dbase
;
;   Input Parameters:
;      time0/time1 - times to include     (required)
;      wavelen - wave(s) to make          (default=171)
;
;   Output Parameters:
;      vdata - summed data
;      vmask - boolean mask
;
;   Keyword Parameters:
;      index  - index records of 
;      minimages - minimum #images to "accept"    (default=10)
;      maximages - maximum #images to sum         (default=50)
;      despike - optional despike - number of iterations (default=3)
;      savedbase - if set, save data to database file 
;      overwrite - if set, overwrite existing dbase file, if any
;      outdir - optional output directory (def=$tdb/vignette)
;
;  History:
;   11-Oct-2000 - S.L.Freeland
;
; Restrictions:
;   currently mask is only a boolean mask
;   TODO - make it a normalization mask
;
;- 
debug=keyword_set(debug)
saveit=keyword_set(savedbase) or keyword_set(outdir) or $
    keyword_set(overwrite)

if not keyword_set(wavelen) then wavelen=171
if not keyword_set(minimages) then minimages=10
if not keyword_set(maximages) then maximages=50

if n_params() lt 2 then begin 
   box_message,'You must supply start and stop time for dbase search'
   return
endif

if n_params() le 2 and (1-saveit) then begin 
   box_message,'You did not choose to save -OR- return data..., exiting'
   return
endif

swave=strtrim(wavelen(0),2)
trace_cat,time0,time1,cat                     ; read catalog

; define parameters of interest
tests=['naxis1=1024',     'naxis2=1024',    $ ; full FOV (vignette)
       'xcen=-600~600',   'ycen=-600~600',  $ ; Avoid Limbs
       'wave_len='+swave, 'percentd=100']     ; Wave / 100% images

sswave=struct_where(cat,test=tests,cnt)          ; apply filters

if cnt ge minimages then begin                        
   good=(1-gtt_orbit(cat(sswave),'IS_SAA')) and $   ; reject SAA
        (1-gtt_orbit(cat(sswave),'IS_HLAT'))        ; reject HLZs
   ssg=where(good,cnt)   
endif

; bailout if not enough images
if cnt lt minimages then begin 
   box_message,['Not enough good images (' +strtrim(minimages,2)+$
                ') ...returning']
   return
endif

; ------ map catlog matches -> data -------

css=last_nelem(sswave(ssg),maximages)
if cnt gt (2*maximages) then $
   css=sswave(ssg(grid_data(cat(sswave(ssg)),nsamp=maximages)))
;  (if #good very much larger thatn MAXIMAGES, then subsample over time)

trace_cat2data, cat(css), index, data                      ; catalog->data

if not keyword_set(quiet) then begin 
   box_message,['Using images:',$
       get_infox(index,'naxis1,naxis2,wave_len,xcen,ycen',fmt_tim='ecs')]
endif

vdata=total(float(data-88.),3)                          ; sum all images
; ------- despike summed image --------
if n_elements(despike) eq 0 then despike=3              ; default=3 iter.
for i=0,despike-1 do begin
   vdata=despike_gen(vdata)                    
   update_history, index, call='despike_gen','Iteration# ' + strtrim(i,2)
endfor
; ---------------------------------------------------

; TODO - cutoff based on 1st peak of histogram or dHIST/dBIN??
acent=average(extract_arr(vdata,512,512,768,768)) ; 100% signal assumed here
vsignal=vdata gt .25*acent                        ; empirical 1st guess
find_limb,vsignal,xx,yy,rr               ; find "limb" with boolean
cirmask=cir_mask(vsignal,xx,yy,rr)       ; circle subscripts
vmask=intarr(1024,1024)                ; 
vmask(cirmask)=1                       ; set boolean

; ------------- optionally save --------------
if saveit then begin
   filename='trace_vignette_'+swave
   case 1 of 
      data_chk(outdir,/string):
      savedbase: begin
         outdir=concat_dir('$tdb','vignette')
         if not write_access(outdir) then begin 
            box_message,['Warning: you asked to update the TRACE dbase',$
                         'via /SAVEDBASE but you dont have the authority',$
                         'saving to current directory...']
            outdir=curdir()
         endif
      endcase
      else: outdir=curdir()
   endcase         
   savegenx, file=concat_dir(outdir,filename), vdata, vmask, index, $
             text='summed data, boolean mask, index records', $
             overwrite=overwrite
endif
; -------------------------------------------

if debug then stop
return
end
